@@ -1,8 +1,51 @@
-`histobarmap` <-
-function(long,lat,var1,var2,criteria=NULL,carte=NULL,label = "",cex.lab=1, pch = 16, col= "grey",
-nbcol = 10,names.arg="", xlab=c("",""), ylab=c("",""), listvar=NULL, listnomvar=NULL,
-axes=FALSE,lablong="", lablat="")
+`histobarmap` <- function(sp.obj, names.var, nbcol = 10, type = "count",
+names.arg = "", names.attr=names(sp.obj), criteria=NULL, carte=NULL, identify=FALSE,
+cex.lab=0.8, pch=16, col="lightblue3", xlab=c("barplot","histogram"), ylab=rep("count",2),
+axes=FALSE, lablong="", lablat="")
 {
+# Verification of the Spatial Object sp.obj
+class.obj<-class(sp.obj)[1]
+
+if(substr(class.obj,1,7)!="Spatial") stop("sp.obj may be a Spatial object")
+if(substr(class.obj,nchar(class.obj)-8,nchar(class.obj))!="DataFrame") stop("sp.obj should contain a data.frame")
+if(!is.numeric(names.var) & length(match(names.var,names(sp.obj)))!=length(names.var) ) stop("At least one component of names.var is not included in the data.frame of sp.obj")
+if(length(names.attr)!=length(names(sp.obj))) stop("names.attr should be a vector of character with a length equal to the number of variable")
+
+# Is there a Tk window already open ?
+if(interactive())
+{
+ if(!exists("GeoXp.open",envir = baseenv())||length(ls(envir=.TkRoot$env, all=TRUE))==2)
+ {
+  assign("GeoXp.open", TRUE, envir = baseenv())
+ }
+ else
+ {if(get("GeoXp.open",envir= baseenv()))
+   {stop("Warning : a GeoXp function is already open. Please, close Tk window before calling a new GeoXp function to avoid conflict between graphics")}
+  else
+  {assign("GeoXp.open", TRUE, envir = baseenv())}
+ }
+}
+
+# we propose to refind the same arguments used in first version of GeoXp
+long<-coordinates(sp.obj)[,1]
+lat<-coordinates(sp.obj)[,2]
+
+var1<-sp.obj@data[,names.var[1]]
+var2<-sp.obj@data[,names.var[2]]
+
+listvar<-sp.obj@data
+listnomvar<-names.attr
+
+ # for colors in map and new grahics
+ ifelse(length(col)==1, col2<-"blue", col2<-col)
+ col3<-"lightblue3"
+
+# Code which was necessary in the previous version
+ if(is.null(carte) & class.obj=="SpatialPolygonsDataFrame") carte<-spdf2list(sp.obj)$poly
+
+
+ # for identifyng the selected sites
+ifelse(identify, label<-row.names(listvar),label<-"")
 
 # initialisation
    nointer<-FALSE
@@ -16,18 +59,16 @@ axes=FALSE,lablong="", lablat="")
    if(names.arg[1]=="") names.arg<-levels(as.factor(var1))
 
    obs <- vector(mode = "logical", length = length(long))
-   fin <- tclVar(FALSE)
    graphChoice <- ""
    varChoice1 <- ""
    varChoice2 <- ""
    choix<-""
    listgraph <- c("Histogram","Barplot","Scatterplot")
       
-# Ouverture des fentres graphiques  
-  graphics.off()
-  dev.new()
-  dev.new()
-  dev.new()
+# Windows device
+if(!(2%in%dev.list())) dev.new()
+if(!(3%in%dev.list())) dev.new()
+if(!(4%in%dev.list())) dev.new()
 
 #transformation data.frame en matrice
 if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(listvar)
@@ -41,6 +82,11 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
     pointfunc <- function() 
     {
         quit <- FALSE
+        
+        dev.set(2)
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
         while (!quit)
          {
             dev.set(2)
@@ -48,18 +94,23 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
             if (is.null(loc)) 
             {
               quit <- TRUE
+              carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
+              cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
+              legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
               next
             }
            obs <<- selectmap(var1 = long, var2 = lat, obs = obs,Xpoly = loc[1], Ypoly = loc[2], method = "point")
            
            carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2, method="Cluster",classe=var1,legmap=legmap,
            legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
-           
+           title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+           title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
            graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot",labvar = labvar1, symbol = pch, 
-           labmod = names.arg,couleurs=col)
+           labmod = names.arg, couleurs=col, bin=type)
            
-           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, labvar = labvar2,couleurs=col[1])
+           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, bin=type, labvar = labvar2,couleurs=col[1])
 
         if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
         {
@@ -78,6 +129,11 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
         polyX <- NULL
         polyY <- NULL
         quit <- FALSE
+        
+        dev.set(2)
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
         while (!quit) 
         {
             dev.set(2)
@@ -101,13 +157,14 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
         obs <<- selectmap(var1=long, var2=lat, obs=obs, Xpoly=polyX, Ypoly=polyY, method="poly")
 
            carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
            legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
            
            graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot",labvar = labvar1, symbol = pch, 
-           labmod = names.arg,couleurs=col)
+           labmod = names.arg, couleurs=col, bin=type)
            
-           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, labvar = labvar2,couleurs=col[1])
+           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, bin=type,
+           labvar = labvar2,couleurs=col[1])
 
         if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
         {
@@ -125,6 +182,11 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
     {
         SGfunc()
         quit <- FALSE
+       
+        dev.set(3)
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
         while (!quit) 
         {
             dev.set(3)
@@ -132,19 +194,24 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
             if (is.null(loc)) 
             {
               quit <- TRUE
+              graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot",labvar = labvar1, symbol = pch, 
+              labmod = names.arg, bin=type,couleurs=col)
               next
             }
 
            obs<<-selectstat(var1=var1,obs=obs,Xpoly=loc[1], Ypoly=loc[2],method="Barplot");    
 
            carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
            legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
            
            graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot",labvar = labvar1, symbol = pch, 
-           labmod = names.arg,couleurs=col)
-           
-           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, labvar = labvar2,couleurs=col[1])
+           labmod = names.arg, couleurs=col, bin=type)
+           title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+           title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
+           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, bin=type,
+           labvar = labvar2,couleurs=col[1])
            
           if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
           {
@@ -162,6 +229,11 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
     {
         SGfunc()
         quit <- FALSE
+       
+        dev.set(4)
+        title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+        title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
+
         while (!quit) 
         {
             dev.set(4)
@@ -169,19 +241,23 @@ if((length(listvar)>0) && (dim(as.matrix(listvar))[2]==1)) listvar<-as.matrix(li
             if (is.null(loc)) 
             {
               quit <- TRUE
+              graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, bin=type, labvar = labvar2,couleurs=col[1])
               next
             }
 
            obs<<-selectstat(var1=var2,obs=obs,Xpoly=loc[1], Ypoly=loc[2],method="Histogram", nbcol=nbcol);   
 
            carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+           cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
            legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
            
            graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot",labvar = labvar1, symbol = pch, 
-           labmod = names.arg,couleurs=col)
+           labmod = names.arg, bin=type, couleurs=col)
            
-           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, labvar = labvar2,couleurs=col[1])
+           graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram", nbcol = nbcol, bin=type,
+           labvar = labvar2,couleurs=col[1])
+           title("ACTIVE DEVICE", cex.main = 0.8, font.main = 3, col.main='red')
+           title(sub = "To stop selection, click on the right button of the mouse and stop (for MAC, ESC)", cex.sub = 0.8, font.sub = 3,col.sub='red')
 
         if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
         {
@@ -201,7 +277,6 @@ graphfunc <- function()
 {
     if ((length(listvar) != 0) && (length(listnomvar) != 0))
     {
-        dev.off(5)
         choix <<- selectgraph(listnomvar,listgraph)
         varChoice1 <<- choix$varChoice1
         varChoice2 <<- choix$varChoice2
@@ -209,7 +284,7 @@ graphfunc <- function()
           
         if ((graphChoice != "") && (varChoice1 != ""))
         {
-            dev.new()
+            if(!(5%in%dev.list())) dev.new()
             graphique(var1=listvar[,which(listnomvar == varChoice1)], var2=listvar[,which(listnomvar == varChoice2)],
             obs=obs, num=5,graph=graphChoice, couleurs=col[1], symbol=pch[1], labvar=c(varChoice1,varChoice2))
             }
@@ -231,7 +306,7 @@ cartfunc <- function()
    {
     ifelse(!nocart,nocart<<-TRUE,nocart<<-FALSE)
     carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-    cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+    cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
     legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
    }
    else
@@ -249,13 +324,14 @@ SGfunc <- function()
    obs <<- vector(mode = "logical", length = length(long))
      
    carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-   cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+   cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
    legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
          
    graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot",labvar = labvar1, symbol = pch, 
-   labmod = names.arg,couleurs=col)
+   labmod = names.arg, bin=type, couleurs=col)
            
-   graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, labvar = labvar2,couleurs=col[1])
+   graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram", nbcol = nbcol, bin=type,
+   labvar = labvar2,couleurs=col[1])
 
    if ((graphChoice != "") && (varChoice1 != "") && (length(dev.list()) > 2))
     {
@@ -269,12 +345,24 @@ SGfunc <- function()
 # quitter l'application
 ####################################################
 
-quitfunc <- function() 
+quitfunc<-function()
 {
- tclvalue(fin) <- TRUE
- tkdestroy(tt)
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+   # assign("obs", row.names(sp.obj)[obs], envir = .GlobalEnv)
 }
 
+quitfunc2<-function()
+{
+    #tclvalue(fin)<<-TRUE
+    tkdestroy(tt)
+    assign("GeoXp.open", FALSE, envir = baseenv())
+    print("Results have been  saved in last.select object")
+    assign("last.select", which(obs), envir = .GlobalEnv)
+}
+
+
 ####################################################
 # Open a no interactive selection
 ####################################################
@@ -285,7 +373,7 @@ fnointer<-function()
  {
   ifelse(!nointer,nointer<<-TRUE,nointer<<-FALSE)
   carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,
-  lablong=lablong, lablat=lablat, label=label, symbol=pch,carte=carte,nocart=nocart,couleurs=col,
+  lablong=lablong, lablat=lablat, label=label, symbol=pch,carte=carte,nocart=nocart,couleurs=col2,
   method="Cluster",classe=var,legmap=legmap,legends=legends,labmod=names.arg,axis=axes,cex.lab=cex.lab)     
  }
  else
@@ -310,7 +398,7 @@ fbubble<-function()
   legmap <<- res2$legmap
   
 carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
 legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
  
 }
@@ -320,13 +408,14 @@ legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
 ####################################################
            
 carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
 legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
            
-graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot",labvar = labvar1, symbol = pch, 
-labmod = names.arg,couleurs=col)
+graphique(var1 = var1, obs = obs, num = 3, graph = "Barplot", bin=type, labvar = labvar1,
+symbol = pch, labmod = names.arg,couleurs=col)
            
-graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, labvar = labvar2,couleurs=col[1])
+graphique(var1 = var2, obs = obs, num = 4, graph = "Histogram",nbcol = nbcol, bin=type,
+labvar = labvar2, couleurs=col[1])
 
 ####################################################
 # cration de la boite de dialogue to create legens
@@ -342,10 +431,11 @@ if(interactive())
 
   dev.set(2)
   loc <- locator(1)
+  loc$name <- names(sp.obj[,names.var[1]])
   legends<<-list(legends[[1]],TRUE,legends[[3]],loc)
 
   carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-  cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+  cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
   legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")
         
  }
@@ -356,7 +446,7 @@ if(interactive())
   tkdestroy(tt1)	
 
   carte(long=long, lat=lat,buble=buble,cbuble=z,criteria=criteria,nointer=nointer,obs=obs,  label=label,
-  cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col,method="Cluster",classe=var1,legmap=legmap,
+  cex.lab=cex.lab,symbol=pch,carte=carte,nocart=nocart,couleurs=col2,method="Cluster",classe=var1,legmap=legmap,
   legends=legends,labmod=names.arg,axis=axes,lablong="", lablat="")        
  }
 
@@ -384,105 +474,79 @@ if(interactive())
 ####################################################
 if(interactive())
 {
+fontheading<-tkfont.create(family="times",size=14,weight="bold")
+
 tt <- tktoplevel()
+tkwm.title(tt, "histobarmap")
 
-labelText1 <- tclVar("Work on the map")
-label1 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(labelText1))
-tkconfigure(label1, textvariable=labelText1)
-tkgrid(label1,columnspan=2)
+frame1a <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1a, text = "Interactive selection", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame1a, text = "Work on the map", font = "Times 12",
+foreground = "darkred", background = "white"))
 
-point.but <- tkbutton(tt, text="   Points   ", command=pointfunc)
-poly.but <- tkbutton(tt, text="   Polygon   ", command=polyfunc)
-tkgrid(point.but, poly.but)
-tkgrid(tklabel(tt,text="    "))
+point.but <- tkbutton(frame1a, text="Selection by point", command=pointfunc);
+poly.but <- tkbutton(frame1a, text="Selection by polygon ", command=polyfunc);
+tkpack(point.but, poly.but, side = "left", expand = "TRUE",fill = "x")
+
+tkpack(frame1a, expand = "TRUE", fill = "x")
+
+frame1c <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame1c, text = "Select bar(s)", font = "Times 12",
+foreground = "darkred", background = "white"))
+barre1.but <- tkbutton(frame1c, text="On barplot ", command=bar1func)
+barre2.but <- tkbutton(frame1c, text="On histogram ", command=bar2func)
+tkpack(barre1.but,barre2.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1c, expand = "TRUE", fill = "x")
 
 
-labelText2 <- tclVar("Work on a graph")
-label2 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText2))
-tkconfigure(label2, textvariable=labelText2)
-tkgrid(label2,columnspan=2)
+frame1b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nettoy.but <- tkbutton(frame1b, text="     Reset selection     " , command=SGfunc);
+tkpack(nettoy.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame1b, expand = "TRUE", fill = "x")
 
 
-barre1.but <- tkbutton(tt, text=" Bar plot ", command=bar1func)
-barre2.but <- tkbutton(tt, text=" Histogram ", command=bar2func)
-tkgrid(barre1.but,barre2.but)
-tkgrid(tklabel(tt,text="    "))
+frame2 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2, text = "Options", font = "Times 14",
+foreground = "blue", background = "white"))
+tkpack(tklabel(frame2, text = "Spatial contours", font = "Times 11",
+foreground = "darkred", background = "white"),tklabel(frame2, text = "Preselected sites", font = "Times 11",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2, expand = "TRUE", fill = "x")
 
+frame2b <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+nocou1.but <- tkbutton(frame2b, text="On/Off", command=cartfunc)
+noint1.but <- tkbutton(frame2b, text="On/Off", command=fnointer)
+tkpack(nocou1.but,noint1.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2b, expand = "TRUE", fill = "x")
 
-label1 <- tclVar("To stop selection, let the cursor on the active graph, click on the right button of the mouse and stop")
-label11 <- tklabel(tt,justify = "center", wraplength = "3i", text=tclvalue(label1))
-tkconfigure(label11, textvariable=label1)
-tkgrid(label11,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+frame2c <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame2c, text = "Bubbles", font = "Times 11",
+foreground = "darkred", background = "white"),tklabel(frame2c, text = "Additional graph", font = "Times 11",
+foreground = "darkred", background = "white"), side = "left", fill="x",expand = "TRUE")
+tkpack(frame2c, expand = "TRUE", fill = "x")
 
+frame2d <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+bubble.but <- tkbutton(frame2d, text="On/Off", command=fbubble)
+autre.but <- tkbutton(frame2d, text="     OK     " , command=graphfunc)
+tkpack(bubble.but,autre.but, side = "left", expand = "TRUE", fill = "x")
+tkpack(frame2d, expand = "TRUE", fill = "x")
 
-labelText7 <- tclVar("Preselected sites")
-label7 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText7))
-tkconfigure(label7, textvariable=labelText7)
-tkgrid(label7,columnspan=2)
+frame3 <- tkframe(tt, relief = "groove", borderwidth = 2, background = "white")
+tkpack(tklabel(frame3, text = "Exit", font = "Times 14",
+foreground = "blue", background = "white"))
 
-noint1.but <- tkbutton(tt, text="  On/Off  ", command=fnointer);
-tkgrid(noint1.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
+quit.but <- tkbutton(frame3, text="Save results", command=quitfunc2);
+quit.but2 <- tkbutton(frame3, text="Exit without saving", command=quitfunc);
 
+tkpack(quit.but, quit.but2, side = "left", expand = "TRUE",
+        fill = "x")
 
-labelText6 <- tclVar("Draw spatial contours")
-label6 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText6))
-tkconfigure(label6, textvariable=labelText6)
-tkgrid(label6,columnspan=2)
+tkpack(frame3, expand = "TRUE", fill = "x")
 
-nocou1.but <- tkbutton(tt, text="  On/Off  ", command=cartfunc);
-tkgrid(nocou1.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-
-
-labelText3 <- tclVar("Restore graph")
-label3 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText3))
-tkconfigure(label3, textvariable=labelText3)
-tkgrid(label3,columnspan=2)
-
-nettoy.but <- tkbutton(tt, text="     OK     " , command=SGfunc);
-tkgrid(nettoy.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-
-labelText9 <- tclVar("  Bubbles  ")
-label9 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText9))
-tkconfigure(label9, textvariable=labelText9)
-tkgrid(label9,columnspan=2)
-
-bubble.but <- tkbutton(tt, text="  On/Off  ", command=fbubble);
-tkgrid(bubble.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-
-labelText4 <- tclVar("Additional graph")
-label4 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText4))
-tkconfigure(label4, textvariable=labelText4)
-tkgrid(label4,columnspan=2)
-
-autre.but <- tkbutton(tt, text="     OK     " , command=graphfunc);
-tkgrid(autre.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-
-labelText5 <- tclVar("  Exit  ")
-label5 <- tklabel(tt,justify = "center", wraplength = "3i",text=tclvalue(labelText5))
-tkconfigure(label5, textvariable=labelText5)
-tkgrid(label5,columnspan=2)
-
-quit.but <- tkbutton(tt, text="     OK     ", command=quitfunc);
-tkgrid(quit.but,columnspan=2)
-tkgrid(tklabel(tt,text="    "))
-
-tkwait.variable(fin);
 }
 
 ####################################################
-# Fin
-####################################################
-   
-return(obs)
+return(invisible())
 }
 
